package icinga

import (
	jsoniter "github.com/json-iterator/go"
)

// EventType struct gets used to get the type of the event we received
// There is probably a more efficient way of doing this
// https://eagain.net/articles/go-dynamic-json/
type EventType struct {
	Type string `json:"type"`
}

type EventTypeCheckResult struct {
	Type        string           `json:"type,omitempty"`
	Timestamp   float64          `json:"timestamp,omitempty"`
	Host        string           `json:"host,omitempty"`
	Service     string           `json:"service,omitempty"`
	CheckResult CheckResultAttrs `json:"check_result"`
}

type EventTypeStateChange struct {
	Type            string           `json:"type,omitempty"`
	Timestamp       float64          `json:"timestamp,omitempty"`
	Acknowledgement bool             `json:"acknowledgement,omitempty"`
	DowntimeDepth   jsoniter.Number  `json:"downtime_depth,omitempty"`
	State           jsoniter.Number  `json:"state,omitempty"`
	StateType       jsoniter.Number  `json:"state_type,omitempty"`
	Host            string           `json:"host,omitempty"`
	Service         string           `json:"service,omitempty"`
	CheckResult     CheckResultAttrs `json:"check_result"`
}

// performance data can be a array of strings or an array of objects.
// I had to learn this the hard way, see icinga_test.go.
// https://icinga.com/docs/icinga-2/latest/doc/08-advanced-topics/#perfdatavalue
type PerformanceDataValue struct {
	Counter bool            `json:"counter,omitempty"`
	Crit    jsoniter.Number `json:"crit,omitempty"`
	Label   string          `json:"label,omitempty"`
	Max     jsoniter.Number `json:"max,omitempty"`
	Min     jsoniter.Number `json:"min,omitempty"`
	Type    string          `json:"type,omitempty"`
	Unit    string          `json:"unit,omitempty"`
	Value   jsoniter.Number `json:"value,omitempty"`
	Warn    jsoniter.Number `json:"warn,omitempty"`
}

type CheckResultAttrs struct {
	Active      bool   `json:"active,omitempty"`
	CheckSource string `json:"check_source,omitempty"`
	// Sometimes command is a string, sometimes an array - wo dont need the check command so i decided to remove it to avoid parser errors
	//Command         []string        `json:"command,omitempty"`
	ExecutionEnd             float64         `json:"execution_end,omitempty"`
	ExecutionStart           float64         `json:"execution_start,omitempty"`
	ExitStatus               jsoniter.Number `json:"exit_status,omitempty"`
	Output                   string          `json:"output,omitempty"`
	PerformanceDataInterface jsoniter.Any    `json:"performance_data,omitempty"`
	ScheduleEnd              float64         `json:"schedule_end,omitempty"`
	ScheduleStart            float64         `json:"schedule_start,omitempty"`
	State                    jsoniter.Number `json:"state,omitempty"`
	Type                     string          `json:"type,omitempty"`
	VarsAfter                *struct {
		Attempt   jsoniter.Number `json:"attempt,omitempty"`
		Reachable bool            `json:"reachable,omitempty"`
		State     jsoniter.Number `json:"state,omitempty"`
		StateType jsoniter.Number `json:"state_type,omitempty"`
	} `json:"vars_after,omitempty"`
	VarsBefore *struct {
		Attempt   jsoniter.Number `json:"attempt,omitempty"`
		Reachable bool            `json:"reachable,omitempty"`
		State     jsoniter.Number `json:"state,omitempty"`
		StateType jsoniter.Number `json:"state_type,omitempty"`
	} `json:"vars_before,omitempty"`
}

type PerformanceDataStr struct {
	PerformanceData []string
}

type PerformanceDataObject struct {
	PerformanceData []PerformanceDataValue
}

type EventAcknowledgementSet struct {
	Type                string          `json:"type"`
	Timestamp           float64         `json:"timestamp"`
	Host                string          `json:"host"`
	Service             string          `json:"service"`
	State               jsoniter.Number `json:"state,omitempty"`
	StateType           jsoniter.Number `json:"state_type"`
	AcknowledgementType jsoniter.Number `json:"acknowledgement_type"`
	Notify              bool            `json:"notify"`
	Expiry              float64         `json:"expiry"`
	Persistent          bool            `json:"persistent"`
	Author              string          `json:"author"`
	Comment             string          `json:"comment"`
}

type EventAcknowledgementCleared struct {
	Type                string          `json:"type"`
	Timestamp           float64         `json:"timestamp"`
	Host                string          `json:"host"`
	Service             string          `json:"service"`
	AcknowledgementType jsoniter.Number `json:"acknowledgement_type,omitempty"`
	State               jsoniter.Number `json:"state,omitempty"`
	StateType           jsoniter.Number `json:"state_type"`
}

type EventDowntimeAdded struct {
	Type      string         `json:"type"`
	Timestamp float64        `json:"timestamp"`
	Downtime  *ObjectComment `json:"downtime"`
}

type EventDowntimeRemoved struct {
	Type      string         `json:"type"`
	Timestamp float64        `json:"timestamp"`
	Downtime  *ObjectComment `json:"downtime"`
}

type EventDowntimeStarted struct {
	Type      string         `json:"type"`
	Timestamp float64        `json:"timestamp"`
	Downtime  *ObjectComment `json:"downtime"`
}

type ObjectComment struct {
	HostName    string          `json:"host_name"`
	ServiceName string          `json:"service_name"`
	Author      string          `json:"author"`
	Comment     string          `json:"comment"`
	StartTime   jsoniter.Number `json:"start_time"`
	EndTime     jsoniter.Number `json:"end_time"`
	Duration    int64           `json:"duration"`
	EntryTime   jsoniter.Number `json:"entry_time"`
	Fixed       bool            `json:"fixed"`
	RemoveTime  jsoniter.Number `json:"remove_time"`
	LegacyId    uint64          `json:"legacy_id"` // Not sure if we should rely on this: https://github.com/Icinga/icinga2/issues/9209#issuecomment-1315133220
}
