package oitc

import (
	"testing"
)

func TestValidHostReturnCodes(t *testing.T) {
	// Up
	rc, err := InHostcheckBoundaries(STATE_UP, "localhost")
	if err != nil {
		t.Error(err)
	}
	if rc != 0 {
		t.Error("Hoststatus needs to be 0!")
	}

	// Down
	rc, err = InHostcheckBoundaries(STATE_DOWN, "localhost")
	if err != nil {
		t.Error(err)
	}
	if rc != 1 {
		t.Error("Hoststatus needs to be 1!")
	}

	// Unreachable
	rc, err = InHostcheckBoundaries(STATE_UNREACHABLE, "localhost")
	if err != nil {
		t.Error(err)
	}
	if rc != 2 {
		t.Error("Hoststatus needs to be 2!")
	}
}

func TestInvalidHostReturnCodes(t *testing.T) {
	var invalidCodes = []int64{3, 4, 5, 10, 100, 1337, -1, -2, -100}
	for _, invalidCode := range invalidCodes {
		rc, err := InHostcheckBoundaries(invalidCode, "localhost")
		if err == nil {
			// We expect an error message
			t.Error(err)
		}
		if rc != 2 {
			t.Error("Hoststatus needs to be 2!")
		}
	}

}

func TestValidServiceReturnCodes(t *testing.T) {
	// Ok
	rc, err := InServicecheckBoundaries(STATE_OK, "localhost", "ping")
	if err != nil {
		t.Error(err)
	}
	if rc != 0 {
		t.Error("Servicestatus needs to be 0!")
	}

	// Warning
	rc, err = InServicecheckBoundaries(STATE_WARNING, "localhost", "ping")
	if err != nil {
		t.Error(err)
	}
	if rc != 1 {
		t.Error("Servicestatus needs to be 1!")
	}

	// Critical
	rc, err = InServicecheckBoundaries(STATE_CRITICAL, "localhost", "ping")
	if err != nil {
		t.Error(err)
	}
	if rc != 2 {
		t.Error("Servicestatus needs to be 2!")
	}

	// Unknown
	rc, err = InServicecheckBoundaries(STATE_UNKNOWN, "localhost", "ping")
	if err != nil {
		t.Error(err)
	}
	if rc != 3 {
		t.Error("Servicestatus needs to be 3!", "ping")
	}
}

func TestInvalidServiceReturnCodes(t *testing.T) {
	var invalidCodes = []int64{4, 5, 10, 100, 1337, -1, -2, -100}
	for _, invalidCode := range invalidCodes {
		rc, err := InServicecheckBoundaries(invalidCode, "localhost", "ping")
		if err == nil {
			// We expect an error message
			t.Error(err)
		}
		if rc != 3 {
			t.Error("Servicestatus needs to be 3!")
		}
	}

}
