package oitc

const SERVICE_CHECK = 0
const HOST_CHECK = 1

const SERVICE_DOWNTIME = 1
const HOST_DOWNTIME = 2

const HOST_ACKNOWLEDGEMENT = 0
const SERVICE_ACKNOWLEDGEMENT = 1

type BrokerCheckResult struct {
	Command string       `json:"Command"` // Always "check_result" to tell the Statusengine broker that is is a check_result to process
	Data    *CheckResult `json:"Data"`
}

// Standard format how openITCOCKPIT expects check results to look like
// https://github.com/statusengine/broker/blob/c21bbf240638e2e40f046d3a73614c48f4452190/src/MessageHandler/MessageHandler.h#L162-L197
type CheckResult struct {
	HostName           string  `json:"host_name"`
	ServiceDescription string  `json:"service_description,omitempty"`
	Output             string  `json:"output"`
	LongOutput         string  `json:"long_output"`
	PerfData           string  `json:"perf_data"`
	CheckType          int64   `json:"check_type"` // 0 = SERVICE_CHECK 1 = HOST_CHECK
	ReturnCode         int64   `json:"return_code"`
	StartTime          int64   `json:"start_time"`
	EndTime            int64   `json:"end_time"`
	EarlyTimeout       int64   `json:"early_timeout"`
	Latency            float64 `json:"latency"`
	ExitedOk           int64   `json:"exited_ok"`
}

type BrokerRawCommand struct {
	Command string `json:"Command"` // Always "raw"
	Data    string `json:"Data"`    // E.g.: 'ACKNOWLEDGE_HOST_PROBLEM;%s;%s;%s;1;%s;%s',
}

type CancelDowntime struct {
	HostUuid     string
	ServiceUuid  string
	Comment      string
	StartTime    int64
	EndTime      int64
	CanceledTime int64
}
